/*
 * State.h
 *
 *  Created on: 21 Nov 2019
 *      Author: webpigeon
 */

#ifndef LIB_STATE_H_
#define LIB_STATE_H_

#include <vector>
#include <string>
#include <unordered_map>

#include "lib/hex.h"

using std::string;
using std::vector;
using std::unordered_map;

typedef Hex location;
typedef unordered_map<string, int32_t> property_map;

struct data {
	int tile;
	int resource;
};

class entity {

	public:
		entity(): tag(0), pos(Hex(0,0,0)) {}
		entity(entity const& rhs): tag(rhs.tag), pos(Hex(0,0,0)) {}
		int32_t get_property(string const& name) const;
		int32_t get_property(string const& name, int32_t val) const;

	private:
		uint64_t tag;
		location pos;
		property_map properties;
};

class state {
	public:
		state();
		virtual ~state();

		entity entity_at(location location);
		vector<entity> entities_at(location location);

		uint32_t terrain_at(location location);
		uint32_t resource_at(location location);

	private:
		vector<entity> entities;
		unordered_map<location,data> map;
		unordered_map<uint8_t, property_map> properties;
		uint32_t turn;
};

#endif /* LIB_STATE_H_ */
